//Meadiator Pattern - http://ru.wikipedia.org/wiki/%D0%9F%D0%BE%D1%81%D1%80%D0%B5%D0%B4%D0%BD%D0%B8%D0%BA_(%D1%88%D0%B0%D0%B1%D0%BB%D0%BE%D0%BD_%D0%BF%D1%80%D0%BE%D0%B5%D0%BA%D1%82%D0%B8%D1%80%D0%BE%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F
"use strict";

var mediator = function () {
	var debug = function () {

        if (typeof console.log !== 'undefined') {
            console.log(arguments[0]);
        }
    };

    var components = {};

    var broadcast = function (event, args, source) {
        var e = event || false;
        var a = args || [];
        if (!e) {
            return;
        }
        if (a.constructor != Array) {
            a = [a];
        }
        debug(["Mediator received", e, a].join(' '));
        for (var c in components) {
            if (typeof components[c]["on" + e] == "function") {
                var s = source || components[c];
                try {
                    debug("Mediator calling " + e + " on " + c);
                    components[c]["on" + e].apply(s, a);
                } catch (err) {
                    debug(["Mediator error.", e, a, s, err].join(' '));
                }
            }
        }
    };

    var addComponent = function (name, component, extend) {
        if (name in components) {
            if (extend) {
                for (var prop in component) {
                    components[name][prop] && debug(prop + ' overwriten!');
                    components[name][prop] = component[prop];
                }
                return;
            } else {
                throw new Error('Mediator name conflict: ' + name);
            }
        }
        components[name] = component;
    };

    var removeComponent = function (name) {
        if (name in components) {
            delete components[name];
        }
    };

    var getComponent = function (name) {
        return components[name] || false;
    };

    var contains = function (name) {
        return (name in components);
    };

    return {
        name: "mediator",
        broadcast: broadcast,
        add: addComponent,
        remove: removeComponent,
        get: getComponent,
        contains: contains
    };
}();